<?php
/**
 * @file
 * Contains the Logger model.
 */

namespace Redis\Logger;


class Writer {

  const PREFIX = 'drupal:logger';

  /**
   * @var \Redis
   *   Assumes use of Predis, not PhpRedis
   */
  protected $redis;

  /**
   * @var \Redis\Logger\Writer
   */
  protected static $instance;

  /**
   * TODO implement
   *
   * @var int
   *   Maximum size of per-template event list. Non-empty() values enable
   *   trimming on insert, so have a tiny performance impact: two O(1) commands
   *   instead of one.
   */
  protected $list_limit = NULL;

  /**
   * @var bool
   *   - true: messages are stored during the page cycle and sent on shutdown.
   *   - false: message are sent immediately
   */
  protected $deferred = FALSE;

  /**
   * @var \Redis\Logger\Entry[]
   */
  protected $entries = array();

  /**
   * Constructor.
   *
   * @param \Redis $redis
   */
  protected function __construct(\Redis $redis) {
    $this->redis = $redis;
  }

  /**
   * Public method to build a Logger from the singleton Redis client.
   *
   * @return static
   */
  public static function createFromGlobals() {
    $redis = \Redis_Client::getClient();
    assert('$redis instanceof \Redis');
    return new static($redis);
  }

  /**
   * @return string[]
   */
  public function getChannels() {
    $templates = $this->getRawTemplates();
  }

  /**
   * @return int[]
   */
  public function getRawTemplates($minimum_level = WATCHDOG_DEBUG, $channel = '*') {
    $this->redis->getKeys(static::PREFIX . ":*:$channel:*");
  }

  /**
   * Logger singleton accessor.
   *
   * @return \Redis\Logger\Writer
   */
  public static function instance() {
    if (!isset(static::$instance)) {
      static::$instance = static::createFromGlobals();
    }

    return static::$instance;
  }

  /**
   * @param \Redis\Logger\Entry $entry
   */
  protected function post(Entry $entry) {

  }

  /**
   * @param \Redis\Logger\Entry $entry
   */
  public function log(Entry $entry) {
    if ($this->deferred) {
      $this->entries[] = $entry;
    }
    else {
      $this->post($entry);
    }
  }
}
